/**************************************************************************************
Author:			David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:		geometry_hard_edges.ms
Date:			2/10/09
Description:	This script will search edit polys/meshes for edge corners. Hard edges
                are defined using 90 deg plus/minus the tolerance.
***************************************************************************************/

try (destroyDialog hard_edge_rollout) catch()


struct HardEdges
(
    /**************************************************************************
    Returns true if the 'current_node' is an edit poly/mesh class and has no
    modifiers applied
    **************************************************************************/
    fn validate_object current_node =
    (
        local is_valid = false
        
        if (classOf current_node) == Editable_Poly or (classOf current_node) == Editable_Mesh do
        (
            if current_node.modifiers.count == 0 do
                is_valid = true
        ) --end if
        
        is_valid --return value
    ), --end fn
    
    /**************************************************************************
    Sets the edge selection on 'poly_node' to all "hard" edges (corners) based
    on the face normal angles
    **************************************************************************/
    fn select_poly_edges poly_node angle_tolerance color_verts =
    (
		local face_edges = #()
		local current_adjacent_faces = #()
		local all_adjacent_faces = #()
		local face_normals = #(0.0,0.0,0.0)
		local all_hard_edges = #()
		
		for x = 1 to (polyop.getNumFaces poly_node) do
		(
			face_edges = polyop.getFaceEdges poly_node x
			
			for edge in face_edges do
			(
				current_adjacent_faces = polyop.getEdgeFaces poly_node edge
                
				if (findItem current_adjacent_faces x) > 0 do
                    deleteItem current_adjacent_faces (findItem current_adjacent_faces x)
                
				join all_adjacent_faces current_adjacent_faces
			)--end for
			
			for y = 1 to all_adjacent_faces.count do
			(
				face_normals[1] = polyop.getFaceNormal poly_node x
				face_normals[2] = polyop.getFaceNormal poly_node all_adjacent_faces[y] 
				face_normals[3] = acos(dot (normalize face_normals[1]) (normalize face_normals[2]))
                
				if face_normals[3] > 360.0 or face_normals[3] < 0 do
                    face_normals[3] = 0
                    
				if face_normals[3] > (90 - angle_tolerance) and face_normals[3] < (90 + angle_tolerance) do
                    appendIfUnique all_hard_edges face_edges[y]
			)--end for
            
            free all_adjacent_faces
		)--end for
		
		polyop.setEdgeSelection poly_node all_hard_edges
		
        if color_verts do
            polyop.setVertColor poly_node 0 (polyop.getVertsUsingEdge poly_node all_hard_edges) (color 0 0 0)
    ), --end fn
    
    /**************************************************************************
    Sets the edge selection on 'mesh_node' to all "hard" edges (corners) based
    on the face normal angles
    **************************************************************************/
    fn select_mesh_edges mesh_node angle_tolerance color_verts =
    (
		local face_edges = #()
		local adjacent_face_edges = #()
		local reverse_face_edges = #()
		local current_adjacent_faces = #()
		local face_normals = #(0.0,0.0,0.0)
		local average_normals = #(0.0,0.0)
		local normals_difference = 0.0
		local all_hard_edges = #()
		
		for i = 1 to (meshop.getNumFaces mesh_node) do
		(
			face_normals = meshop.getFaceRNormals mesh_node i
			average_normals[1] = (face_normals[1] + face_normals[2] + face_normals[3]) / 3.0
			face_edges = meshop.getEdgesUsingFace mesh_node i
			reverse_face_edges = meshop.getEdgesReverseEdge mesh_node face_edges
			current_adjacent_faces = (meshop.getFacesUsingEdge mesh_node reverse_face_edges) as array
			
			for x = 1 to current_adjacent_faces.count do
			(
				face_normals = meshop.getFaceRNormals mesh_node current_adjacent_faces[x]
				average_normals[2] = (face_normals[1] + face_normals[2] + face_normals[3]) / 3.0
				normals_difference = acos(dot average_normals[1] average_normals[2])
				
				if normals_difference > 360.0 do
                    normals_difference = 0
				
				if normals_difference > (90 - angle_tolerance) and normals_difference < (90 + angle_tolerance) do
				(
					adjacent_face_edges = (meshop.getEdgesUsingFace mesh_node current_adjacent_faces[x]) as array
					
					for y = 1 to adjacent_face_edges.count do
					(
						reverse_face_edges = (meshop.getEdgesReverseEdge mesh_node adjacent_face_edges[y]) as array
						
						if reverse_face_edges.count > 0 do
						(
							if (findItem face_edges reverse_face_edges[1]) > 0 do
							(
								appendIfUnique all_hard_edges reverse_face_edges[1]
								appendIfUnique all_hard_edges adjacent_face_edges[y]
								y = adjacent_face_edges.count
							)--end if
						)--end if
					)--end for
				)--end if
			)--end for
		)--end for
		
		setEdgeSelection mesh_node all_hard_edges
        
		if color_verts do
            meshop.setVertColor mesh_node 0 (meshop.getVertsUsingEdge mesh_node all_hard_edges) (color 0 0 0)
    ) --end fn
) --end struct


(--put script into local scope
	global hard_edge_rollout
	local hard_edge_ini_file = getDir #plugcfg + "\\HardEdge.ini"
	local hard_edge_rollout_pos
	
	rollout hard_edge_rollout "Find Hard Edges"
	(
        local hard_edges = HardEdges()
        
        groupBox angle_group "Angle Tolerance" pos:[5,5] width:190 height:40
            label tolerance_label "90 Degrees +/- :" pos:[15,25] width:90 height:15
            spinner tolerance_spinner "" pos:[110,24] width:45 height:15 range:[0.1,89.0,20.0] type:#float scale:0.1
        
        groupBox color_group "Color" pos:[5,50] width:190 height:65
            checkbox color_vertex_checkbox "Set Vertex Color" pos:[15,70] width:105 height:15
            button reset_vertex_color_button "Reset Vertex Colors Now" pos:[15,90] width:170 height:20
        
        groupBox options_group "Options" pos:[5,120] width:190 height:40
            checkbox selection_only_checkbox "Selection Only" pos:[15,140] checked:true
        
		progressbar prog_bar pos:[5,170] width:190 height:10
		button find_hard_edge_button "Find Hard Edges" pos:[5,185] width:190 height:20
		label status_label "INFO: Standing by..." pos:[5,215] width:190 height:15	
		
        /**********************************************************************
        UI move event handler. Stores the current position of the UI
        **********************************************************************/
		on hard_edge_rollout moved pos do
            setIniSetting hard_edge_ini_file "Dialog" "Position" (pos as string)
		
        /**********************************************************************
        Reset colors event handler. Resets the vertex colors for selected/all
        geometry
        **********************************************************************/
		on reset_vertex_color_button pressed do
		(
            local geometry_objects = #()
			local error_counter = 0
            local prog_bar_color = color 200 0 0
			
			prog_bar.value = 0
			prog_bar.color = prog_bar_color
            
            status_label.text = "INFO: Resetting colors. Please wait..."
            if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
			
			if selection_only_checkbox.checked then
                geometry_objects = selection as array
			else
                geometry_objects = geometry
			
			for current in geometry_objects do
			(
				if (hard_edges.validate_object current) then
                (
                    if (classOf current) == Editable_Poly then
                        polyop.setVertColor current 0 #{1..(polyop.getNumVerts current)} (color 255 255 255)
                    else
                        meshop.setVertColor current 0 #{1..(meshop.getNumVerts current)} (color 255 255 255)
                ) --end if
                else
                (
                    print ("ERROR: Object must be an edit poly/mesh with no modifiers. Cannot operate on: " + current.name)
                    error_counter += 1
                ) --end else
			)--end for
			
            if error_counter == 0 then
            (
                status_label.text = ("INFO: Vertex color reset")
                prog_bar_color = color 0 200 0
            ) --end if
            else
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s) occurred. Check listener")
            
            prog_bar.value = 100
            prog_bar.color = prog_bar_color
		)--end on
		
        /**********************************************************************
        Find edge event handler. Searches the selected/all geometry for hard
        edges (corners)
        **********************************************************************/
		on find_hard_edge_button pressed do
		(
            local geometry_objects = #()
            local percent = 0
			local error_counter = 0
            local prog_bar_color = color 200 0 0
			
			prog_bar.value = 0
			prog_bar.color = prog_bar_color
            
            status_label.text = "INFO: Searching objects. Please wait..."
            if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
			
			if selection_only_checkbox.checked then
                geometry_objects = selection as array
			else
                geometry_objects = geometry
                
            for i = 1 to geometry_objects.count do
			(
				if (hard_edges.validate_object geometry_objects[i]) then
                (
                    if (classOf geometry_objects[i]) == Editable_Poly then
                        hard_edges.select_poly_edges geometry_objects[i] tolerance_spinner.value color_vertex_checkbox.checked
                    else
                        hard_edges.select_mesh_edges geometry_objects[i] tolerance_spinner.value color_vertex_checkbox.checked
                ) --end if
                else
                (
                    print ("ERROR: Object must be an edit poly/mesh with no modifiers. Cannot operate on: " + geometry_objects[i].name)
                    error_counter += 1
                ) --end else
                
                percent = 100.0 * i/geometry_objects.count
				prog_bar.value = percent
				if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
			)--end for
			
            if error_counter == 0 then
            (
                status_label.text = ("INFO: Operation complete")
                prog_bar_color = color 0 200 0
            ) --end if
            else
                status_label.text = ("WARNING: " + (error_counter as string) + " error(s). Check listener")
            
            prog_bar.value = 100
            prog_bar.color = prog_bar_color
		)--end on
	)--end rollout
    
	hard_edge_rollout_pos = execute (getIniSetting hard_edge_ini_file "Dialog" "Position")
	if hard_edge_rollout_pos == OK do hard_edge_rollout_pos = [100,100]
	if hard_edge_rollout_pos.x > sysinfo.desktopsize.x - 200 do
        hard_edge_rollout_pos.x = sysinfo.desktopsize.x - 205
	if hard_edge_rollout_pos.y > sysinfo.desktopsize.y - 290 do
        hard_edge_rollout_pos.y = sysinfo.desktopsize.y - 290
	if hard_edge_rollout_pos.x < 0 do hard_edge_rollout_pos.x = 0 
	if hard_edge_rollout_pos.y < 0 do hard_edge_rollout_pos.y = 0
	createDialog hard_edge_rollout 200 230 hard_edge_rollout_pos.x hard_edge_rollout_pos.y
)--end local scope
